REBOL [
	Title: "REBOL/Services Client CLI"
	Version: 1.0.0
	Author: "Gabriele Santilli"
]

;lns-url: http://www.rebol.net/cgi-bin/basic-cgi.r
lns-url: tcp://localhost:8000
lns-port: none

do %rebser-load-client.r
;do http://www.rebol.net/rebservices/client.r
ctx-services/debug-enable: false

print [system/script/header/title system/script/header/version]
print "Type HELP for more information."

help-menu: {
Local Commands:
	   QUIT                - quit shell
	   LOGIN [user [pass]] - login as user
	   LOGOUT              - logout
	   HELP                - this help
	   RESET               - reset connection
	   OPEN url            - connect to different server
	   LOCAL expr          - evaluate expr locally

Useful Remote Commands:
	   INFO
	   COMMANDS
	   DATE
	   SYSTEM
	   LANGUAGE

Any other command is sent to the server, along with arguments. For example:

	   ECHO "How ya doing?"
	   ECHO [%file $10.00 http://www.rebol.com]
}

if-error:
	func [
		"Tries to DO code, then DOes on-error if it fails" [throw]
		code [block!]
		on-error [block!] "The word 'error will refer to the error"
		
		/local result
	] [
		on-error: func [[throw] error [error!]] on-error
		either error? set/any 'result try code [on-error :result] [get/any 'result]
	]

form-error:
	func [
		"Forms an error message"
		errobj [object!] "Disarmed error"
		/all "Use the same format as the REBOL console"
		
		/local errtype text
	] [
		errtype: get in system/error get in errobj 'type
		text: get in errtype get in errobj 'id
		if block? text [text: reform bind/copy text in errobj 'self]
		either all [
			rejoin [
				"** " get in errtype 'type ": " text newline
				either get in errobj 'where [join "** Where: " [mold get in errobj 'where newline]] [""]
				either get in errobj 'near [join "** Near: " [mold/only get in errobj 'near newline]] [""]
			]
		] [
			text
		]
	]

lns-port: open-service lns-url

do-local:
	func [cmd] [
		bind cmd system/words
		set/any 'cmd if-error cmd [
			prin form-error/all disarm error
			exit
		]
		if value? 'cmd [print ["L===" mold cmd]]
	]

reset-block: [close-service lns-port lns-port: open-service lns-url ()]

forever [
	request: to block! ask "LNS> "
	if not parse request [
		['q | 'quit] (quit)
		|
		'login set user opt string! set pass opt string! (
			user: any [user ask "Username? "]
			pass: any [pass ask/hide "Password? "]
			if not login-service lns-port user pass [
				print "***Unable to log in!"
			]
		)
		|
		'logout (logout-service lns-port)
		|
		'reset (do-local reset-block)
		|
		'open set lns-url url! (do-local reset-block)
		|
		['! | 'local] copy cmd to end (do-local cmd)
		|
		'help (print help-menu)
		|
		end ; empty command line, ignore
	] [
		if-error [
			result: do-service lns-port request
			either all [result/1 = 'done result/3 = 'ok] [
				print ["====" mold/only next result/4]
			] [
				print ["!!!!" mold result]
			]
		] [
			prin form-error/all disarm error
		]
	]
]
